"
    Copyright (c) 2012-2016 Igor Stasenko
                            Martin McClure
                            Damien Pollet
                            Camillo Bruni
                            Guido Chari
                   2016-now Jan Vrany <jan.vrany [at] fit . cvut . cz>

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the 'Software'), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"
"{ Package: 'jv:dragonfly/asm' }"

"{ NameSpace: Smalltalk }"

AJInstruction subclass:#AJAlignmentInstruction
	instanceVariableNames:'alignTo'
	classVariableNames:''
	poolDictionaries:'AJConstants'
	category:'AsmJit-Instructions'
!

AJAlignmentInstruction comment:'I am a pseudo instruction used to align the following instruction to a multiple of a given byte number.

Example:
	asm := AJx64Assembler noStackFrame.
	
	"align the following instruction to a word (2bytes)"
	asm alignWord.
	asm inc: asm RAX.
	
	"align the following instruction to a double (4bytes)"
	asm alignDouble.
	asm inc: asm RAX.
	
	"align the following instruction to a QuadWord (8bytes)"
	asm alignQuad.
	asm inc: asm RAX.
	
	"align the following instruction to a multiple of an arbirary count"
	asm align: 64.
	asm inc: asm RAX.'
!

!AJAlignmentInstruction class methodsFor:'documentation'!

copyright
"
    Copyright (c) 2012-2016 Igor Stasenko
                            Martin McClure
                            Damien Pollet
                            Camillo Bruni
                            Guido Chari
                   2016-now Jan Vrany <jan.vrany [at] fit . cvut . cz>

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the 'Software'), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

"
! !

!AJAlignmentInstruction class methodsFor:'instance creation'!

align: byteSize
    ^ self new align: byteSize
!

alignDouble
    ^ self new alignDouble
!

alignQuad
    ^ self new alignQuad
!

alignWord
    ^ self new alignWord
! !

!AJAlignmentInstruction methodsFor:'accessing'!

align
    ^ alignTo
!

align: bytesSize
    "align the data to the given byte count"
    alignTo := bytesSize
!

name
    ^ String streamContents: [ :s|
        self printSelfOn: s]
!

printSelfOn: aStream  
    self align <= 1 ifTrue: [ ^ self ].
    
    aStream nextPut: $|.
    self align <= 8 
        ifTrue: [
            self align timesRepeat: [
                aStream nextPutAll: '----|']]
        ifFalse: [
            aStream 
                nextPutAll: (self align asString padded: #left to: 4 with: $ );
                nextPut: $|]
! !

!AJAlignmentInstruction methodsFor:'alignment'!

alignByte
    self align: 1
!

alignDouble
    self align: 4
!

alignQuad
    self align: 8
!

alignWord
    self align: 2
! !

!AJAlignmentInstruction methodsFor:'emitting code'!

emitCode: asm
    | padding |
    
    padding := self paddingForPosition: position.
    
    "new machine code: | padding |"
    machineCode := ByteArray new: padding .
!

paddingForPosition: aPositionNumber
    | padding |
    
    padding := aPositionNumber \\ self align.
    padding = 0 
        ifFalse: [ padding := self align - padding ].
        
    ^ padding
! !

!AJAlignmentInstruction methodsFor:'initialize-release'!

initialize 
    super initialize.
    self alignByte.
! !

!AJAlignmentInstruction methodsFor:'visitor'!

accept: anObject
    self shouldBeImplemented 
! !

