"
    Copyright (c) 2012-2016 Igor Stasenko
                            Martin McClure
                            Damien Pollet
                            Camillo Bruni
                            Guido Chari
                   2016-now Jan Vrany <jan.vrany [at] fit . cvut . cz>

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the 'Software'), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"
"{ Package: 'jv:dragonfly/asm' }"

"{ NameSpace: Smalltalk }"

AJx86InstructionDescription subclass:#AJx64InstructionDescription
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'AsmJit-x86-Instructions'
!

!AJx64InstructionDescription class methodsFor:'documentation'!

copyright
"
    Copyright (c) 2012-2016 Igor Stasenko
                            Martin McClure
                            Damien Pollet
                            Camillo Bruni
                            Guido Chari
                   2016-now Jan Vrany <jan.vrany [at] fit . cvut . cz>

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the 'Software'), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.

"
! !

!AJx64InstructionDescription methodsFor:'code emitting'!

emitalu: emitter operand1: op1 operand2: op2 operand3: op3
    | opCode opReg |
    opCode := opCode1.
    opReg := opCodeR.	" Mem <- Reg "
    (op1 isMem and: [ op2 isReg ])
        ifTrue: [ 
            ^ emitter
                emitX86RM: opCode + op2 isRegTypeGPB not asBit
                size: op2 size
                regOrCode: op2
                rm: op1 ].	"Reg <- Reg|Mem"
    (op1 isReg and: [ op2 isRegMem ])
        ifTrue: [ 
            (op2 isReg and: [ op1 size ~= op2 size ])
                ifTrue: [ self invalidInstruction ].
            ^ emitter
                emitX86RM: opCode + 2 + op1 isRegTypeGPB not asBit
                size: op1 size
                regOrCode: op1
                rm: op2 ].
    op2 isImm
        ifFalse: [ self invalidInstruction ].	" AL, AX, EAX, RAX register shortcuts"
    (op1 isRegIndex: 0)
        ifTrue: [ 
            op1 is16
                ifTrue: [ emitter emitByte: 16r66	" 16 bit " ].
            op1 is64
                ifTrue: [ emitter emitByte: 16r48	" REX.W" ].
            emitter emitByte: (opReg << 3 bitOr: 16r04 + op1 isRegTypeGPB not asBit).
            ^ emitter emitImmediate: op2 size: (op1 size min: 4) ].	"short constant"
    op2 isInt8
        ifTrue: [ 
            | szBits |
            szBits := op1 size = 1
                ifTrue: [ 0 ]
                ifFalse: [ 3 ].
            emitter
                emitX86RM: opCode2 + szBits
                size: op1 size
                regOrCode: opReg
                rm: op1
                immSize: 1.
            ^ emitter emitImmediate: op2 size: 1 ].
    op1 isRegMem
        ifTrue: [ 
            | immSize szBits |
            immSize := op2 isInt8
                ifTrue: [ 1 ]
                ifFalse: [ op1 size min: 4 ].
            szBits := op1 size ~= 1
                ifTrue: [ 
                    immSize ~= 1
                        ifTrue: [ 1 ]
                        ifFalse: [ 3 ] ]
                ifFalse: [ 0 ].
            emitter
                emitX86RM: opCode2 + szBits
                size: op1 size
                regOrCode: opReg
                rm: op1
                immSize: immSize.
            ^ emitter emitImmediate: op2 size: immSize ].
    self invalidInstruction
!

emitbswap: emitter operand1: op1 operand2: op2 operand3: op3

    op1 isReg ifTrue: [
        emitter emitRexForSingleOperand: op1.
        emitter emitByte: 16r0F.
        ^ emitter emitModR: 1 r: op1 code
    ].
    self invalidInstruction.
!

emitcall: emitter operand1: op1 operand2: op2 operand3: op3

    (op1 isReg and: [ op1 is64 and: [ op1 isUpperBank not ]]) ifTrue: [ 
        "shortcut"
        emitter emitByte: 16rFF. 
        op1 emitModRM: emitter code: 2 immSize: 4.
        ^ self].
    
    (op1 isMem and: [op1 hasBase and: [ op1 base is64 and: [ op1 base isUpperBank not ]]]) ifTrue: [ 
        "shortcut"
        emitter emitByte: 16rFF. 
        op1 emitModRM: emitter code: 2 immSize: 4.
        ^ self].
    
    (op1 isMem or: [ op1 isReg and: [ op1 is64 ] ]) ifTrue: [
        ^ emitter emitX86RM:  16rFF
            size: 4
            regOrCode: 2  
            rm: op1  
    ].

    op1 isImm ifTrue: [ "call by relative offset, you should be really sure what you're' doing"
        emitter emitByte: 16rE8. 
        op1 emitUsing: emitter size: 4.
        ^ self.
        ].
    
    op1 isLabel ifTrue: [
        emitter emitByte: 16rE8. 
        emitter emitDisplacement: op1 inlinedDisp: -4.
        ^ self
    ].
    self invalidInstruction.
!

emitmov: emitter operand1: dst operand2: src operand3: op3
    
    src isReg & dst isReg ifTrue: [
        self assert: (src isRegTypeGPB | src isRegTypeGPW | src isRegTypeGPD | src isRegTypeGPQ ).
        ]. 
    
    " reg <- mem "
    dst isReg & src isRegMem ifTrue: [
        self assert: (dst isRegTypeGPB | dst isRegTypeGPW | dst isRegTypeGPD | dst isRegTypeGPQ ).
        src isMem
            ifTrue: [ (src base notNil and: [ src base is64 not ])
                ifTrue: [ Error signal: 'use a 64bit base register instead of ', src base asString, '(', (src base size * 8) asString, 'bit) for memory access on a 64bit CPU' ]]
            ifFalse:[ (src size = dst size) 
                ifFalse: [ Error signal: 'source ',src asString,' and destination ',dst asString,' need to have the same size' ]].
        ^ emitter emitX86RM: 16r0000008A + dst isRegTypeGPB not asBit 
            size:  dst size 
            regOrCode:  dst 
            rm: src
        ].

    " reg <- imm "
    dst isReg & src isImm ifTrue: [
        | immSize |
        immSize := dst size.
        "Optimize instruction size by using 32 bit immediate if value can fit to it"
        emitter is64BitMode & immSize = 8 & src isInt32 & (src relocMode == #RelocNone) ifTrue: [
             emitter emitX86RM: 16rC7
                size: dst size
                regOrCode: 0 rm: dst.
            immSize := 4
        ] ifFalse: [
            emitter emitX86Inl: (immSize=1 ifTrue: [16rB0] ifFalse: [16rB8]) reg: dst.
        ].
        ^ emitter emitImmediate: src size: immSize
    ].

    "mem <- reg"
    dst isMem & src isReg ifTrue: [
        self assert: (src isRegTypeGPB | src isRegTypeGPW | src isRegTypeGPD | src isRegTypeGPQ ).
        ^ emitter emitX86RM: 16r88 + src isRegTypeGPB not asBit
            size: src size regOrCode: src rm: dst
    ].

    "mem <- imm"
    dst isMem & src isImm ifTrue: [ | immSize |
        
        immSize := dst size <= 4 ifTrue: [ dst size ] ifFalse: [4].
        
        emitter emitX86RM: 16rC6 + ((dst size = 1) not) asBit
            size: dst size 
            regOrCode:  0 rm:  dst 
            immSize: immSize.
        
        ^ emitter emitImmediate: src size: immSize
    ].

    self invalidInstruction 
!

emitmovPtr: emitter operand1: op1 operand2: op2 operand3: op3

    | reg imm opCode |
    (op1 isReg & op2 isImm) | (op1 isImm & op2 isReg) ifFalse: [
        self invalidInstruction ].
    
    opCode := op1 isReg 
        ifTrue: [reg := op1. imm := op2. 16rA0] 
        ifFalse: [reg := op2. imm := op1. 16rA2].
    
    reg index ~= 0 ifTrue: [ self invalidInstruction ].

    reg isRegTypeGPW ifTrue: [ emitter emitByte: 16r66 ].

    emitter emitRexForSingleOperand: reg.
    emitter emitByte: opCode + (reg size ~=1) asBit.
    emitter emitImmediate: imm size: reg size
!

emitmovSxZx: emitter operand1: dst operand2: src operand3: op3
    dst isReg & src isRegMem
        ifFalse: [ self invalidInstruction ].
    src size >= dst size
        ifTrue: [ self invalidInstruction ].
    dst isGeneralPurpose
        ifFalse: [ self invalidInstruction ].
    src is16
        ifTrue: [ 
            ^ emitter
                emitX86RM: opCode1 + 1
                size: dst size
                regOrCode: dst
                rm: src ].
    src is32
        ifTrue: [ self invalidInstruction ].	"64 bit source"
    emitter
        emitX86RM: opCode1
        size: dst size
        regOrCode: dst
        rm: src
!

emitpop: emitter operand1: op1 operand2: op2 operand3: op3
    op1 isReg ifTrue: [
        self assert: op1 isGeneralPurpose.
        (op1 is32 or: [ op1 is8 ]) 
            ifTrue: [ Error signal: 'invalid register ', op1 name, '. push/pop requires 64bit/16bit reg in 64bit mode'].
        ^ emitter emitX86Inl: opCode1 reg: op1 withRex: op1 isUpperBank.
        ].
    
    op1 isMem ifFalse: [ self invalidInstruction ].
    emitter emitX86RM: opCode2 size: op1 size regOrCode: opCodeR rm: op1
! !

